from collections import OrderedDict

from transformers.modeling_albert import AlbertConfig, AlbertForMaskedLM, AlbertModel, AlbertForQuestionAnswering, \
    AlbertForTokenClassification, AlbertPreTrainedModel, AlbertForSequenceClassification
from transformers.tokenization_albert import AlbertTokenizer

from ..model_base import TaskModels, ModelType, ModelTaskType


class Albert_Task_Models(TaskModels):
    MODEL_TYPE = ModelType.albert
    CONFIG = AlbertConfig
    MODEL_PATHS = {
        "cn": ["clue/albert_chinese_tiny",
               "voidful/albert_chinese_tiny",
               "clue/albert_chinese_small",
               "voidful/albert_chinese_small",
               "onePatient/albert_chinese_small",
               "voidful/albert_chinese_base",
               "voidful/albert_chinese_large"
               ]}
    MODEL_CLASSES = OrderedDict([
        (ModelTaskType.base, AlbertModel),
        (ModelTaskType.lm_head, AlbertForMaskedLM),
        (ModelTaskType.seq_cls, AlbertForSequenceClassification),
        (ModelTaskType.token_cls, AlbertForTokenClassification),
        (ModelTaskType.qa, AlbertForQuestionAnswering),
    ])
    TOKENIZERS = OrderedDict([
        ('default', AlbertTokenizer)
    ])



import json
import os
import random
from typing import Dict

from sklearn.metrics import f1_score

from .dataprocessor import DataProcessor

from .dataset import TaskDataset
import numpy as np
import torch
from ai_harness.fileutils import join_path
from transformers import AutoConfig, AutoTokenizer, EvalPrediction, \
    PreTrainedModel
from transformers.data.metrics import simple_accuracy

from .configuration import ModelArguments, DataArguments, log, Model_Mode, TaskArguments, parse_args
from .models import Model_Tools, Model
from .models import model_class as get_model_class
from .trainer import Trainer
from .trainer_utils import PredictionOutput
from .training_args import TrainingArguments


def set_seed(seed: int):
    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)
    # ^^ safe to call this function even if cuda is not available


##TODO: need to refactor the TaskModel for various cases
##(1) train a pretrained model from scratch
##(2) finetuning a task based on a pretrained Model
##(3) resume a task from the training interruption
## Currently, if need to resume the training, you must reset the model_path to the checkpoint
class TaskModel:
    def __init__(self, modelArgs: ModelArguments, model_class=None):
        self._model_args = modelArgs
        self._model_class = model_class if model_class else modelArgs.model_name
        self.model_path = self._make_model_path()
        self._init()

    def _init(self):
        self._model_args.validate()
        log.info("The model_path is made as: " + str(self.model_path))

        self.config = AutoConfig.from_pretrained(
            self.model_path,
            num_labels=self._model_args.num_labels
        )

        self.tokenizer = AutoTokenizer.from_pretrained(self.model_path)

        self.model = self._model(self.config)

        log.info(
            "Loaded task model, config: {}, tokenizer: {}, model: {} ".format(type(self.config), type(self.tokenizer),
                                                                              type(self.model)))

        if self._model_args.freeze_parameter and self._model_args.freeze_parameter != 'none':
            parameters = eval("self.model." + self._model_args.freeze_parameter + '.parameters()')

            for param in parameters:
                param.requires_grad = False

        log.info("num params:" + str(self.model.num_parameters()))
        log.info("num trainable params:" + str(self.model.num_parameters(only_trainable=True)))

        log.info("Model Parameters Details: ")
        for name, param in self.model.named_parameters():
            log.info("{}:{}".format(name, param.size()))

    def _model(self, config):
        model_class = self._get_model_class()

        return model_class.from_pretrained(
            self.model_path,
            config=config
        )

    def _make_model_path(self):
        if self._model_args.model_path:
            return join_path(self._model_args.model_base_dir, self._model_args.model_path)

        if isinstance(self._model_class, Model):
            return join_path(self._model_args.model_base_dir, self._model_class.path)

        if isinstance(self._model_class, str):
            return join_path(self._model_args.model_base_dir, Model_Tools.model_by(self._model_class).path)

        raise ValueError("Cannot get model path.")

    def _get_model_class(self):
        if isinstance(self._model_class, PreTrainedModel):
            return self._model_class
        model_class = get_model_class(self.config, self._model_args.model_task_type)
        if not model_class:
            if not self._model_class:
                raise ValueError(
                    "Cannot find the model class for model type {} and config {}.".format(
                        self._model_args.model_task_type,
                        self._model_args.model_path)
                )
        return model_class

    def _acc_and_f1(self, preds, labels):
        acc = simple_accuracy(preds, labels)
        f1 = f1_score(y_true=labels, y_pred=preds, average="weighted")
        return {
            "acc": acc,
            "f1": f1,
            "acc_and_f1": (acc + f1) / 2,
        }

    def compute_metrics(self, p: EvalPrediction) -> Dict:
        if self._model_args.model_mode == Model_Mode.classification:
            preds = np.argmax(p.predictions, axis=1)
        elif self._model_args.model_mode == Model_Mode.regression:
            preds = np.squeeze(p.predictions)
        return self._acc_and_f1(preds, p.label_ids)


class TaskData:
    def __init__(self, dataArgs: DataArguments,
                 tokenizer, dataProcessor: DataProcessor, local_rank):
        self._data_args = dataArgs
        self._tokenizer = tokenizer
        self._processor = dataProcessor
        self._local_rank = local_rank
        self._train_data, self._eval_data = None, None

    def train_data(self):
        if not self._train_data:
            self._train_data = TaskDataset(self._data_args,
                                           tokenizer=self._tokenizer,
                                           processor=self._processor,
                                           local_rank=self._local_rank)
        return self._train_data

    def eval_data(self):
        if not self._eval_data:
            self._eval_data = TaskDataset(self._data_args,
                                          tokenizer=self._tokenizer,
                                          processor=self._processor,
                                          local_rank=self._local_rank,
                                          evaluate=True)
        return self._eval_data


class TaskTrainer:
    def __init__(self, trainingArgs: TrainingArguments,
                 taskModel: TaskModel, taskData: TaskData, compute_metrics=None):
        self._training_args = trainingArgs
        self._taskModel = taskModel
        self._taskData = taskData
        self._compute_metrics = compute_metrics if compute_metrics else taskModel.compute_metrics
        self._trainer = self._create_trainer()

    def _check_train_args(self):
        if (
                self._training_args.do_train
                and not self._training_args.overwrite_output_dir
                and os.path.exists(self._training_args.output_dir)
                and os.listdir(self._training_args.output_dir)
        ):
            raise ValueError(
                f"Output directory ({self._training_args.output_dir}) already exists and is not empty. Use --overwrite_output_dir to overcome."
            )

    def _create_trainer(self):
        return Trainer(
            model=self._taskModel.model,
            args=self._training_args,
            train_dataset=self._taskData.train_data() if self._training_args.do_train else None,
            eval_dataset=self._taskData.eval_data() if self._training_args.do_eval else None,
            compute_metrics=self._compute_metrics,
        )

    def train(self):
        if self._training_args.do_train:
            # Here, the model_path is for the trainer to load the optimizer and scheduler states
            self._trainer.train(
                model_path=self._taskModel.model_path
            )
            self._trainer.save_model()
            # For convenience, we also re-save the tokenizer to the same directory,
            # so that you can share your model easily on huggingface.co/models =)
            if self._trainer.is_world_master():
                self._taskModel.tokenizer.save_pretrained(self._training_args.output_dir)
        return self

    def eval(self):
        # Evaluation
        results = {}
        if self._training_args.do_eval and self._training_args.local_rank in [-1, 0]:
            log.info("*** Evaluate ***")

            eval_datasets = [self._taskData.eval_data()]

            for eval_dataset in eval_datasets:
                result = self._trainer.evaluate(eval_dataset=eval_dataset)

                output_eval_file = join_path(
                    self._training_args.output_dir, f"eval_results_{eval_dataset.args.task_name}.txt"
                )
                with open(output_eval_file, "w") as writer:
                    log.info("***** Eval results {} *****".format(eval_dataset.args.task_name))
                    for key, value in result.items():
                        log.info("  %s = %s", key, value)
                        writer.write("%s = %s\n" % (key, value))

                results.update(result)
        return results

    def predict(self) -> PredictionOutput:
        return self._trainer.predict(test_dataset=self._taskData.eval_data())


class TransformerTask:
    def __init__(self, task_args: TaskArguments,
                 dataProcessor, model_class=None, compute_metric=None):
        log.info("Create a TransformerTask with arguments: ")
        log.info(json.dumps(task_args, default=lambda obj: obj.__dict__, indent=True))

        self.task_args = task_args
        self.task_args.training_args.validate()
        self._taskModel = TaskModel(task_args.model_args, model_class)
        self._taskData = TaskData(task_args.data_args, self._taskModel.tokenizer, dataProcessor,
                                  task_args.training_args.local_rank)
        self._taskTrainer = TaskTrainer(task_args.training_args, self._taskModel,
                                        self._taskData, compute_metric)
        self._training_args = task_args.training_args

    def _log_task_start(self):
        log.warning(
            "Process rank: %s, device: %s, n_gpu: %s, distributed training: %s, 16-bits training: %s",
            self._training_args.local_rank,
            self._training_args.device,
            self._training_args.n_gpu,
            bool(self._training_args.local_rank != -1),
            self._training_args.fp16,
        )
        log.info("Training/evaluation parameters %s", self._training_args)

    def train(self):
        if not self._training_args.do_train:
            log.warn("do_train argument is set for False")
            return
        self._log_task_start()

        set_seed(self._training_args.seed)
        results = self._taskTrainer.train().eval()
        return results

    def eval(self):
        self._log_task_start()
        set_seed(self._training_args.seed)
        results = self._taskTrainer.eval()
        return results

    def single_predict(self, *input):
        return self._taskModel.model(*input)

    def predict(self) -> PredictionOutput:
        return self._taskTrainer.predict()


class DefaultTask(TransformerTask):
    def __init__(self, dataProcessorClass, model_class=None, compute_metric=None):
        super().__init__(parse_args(), dataProcessorClass, model_class, compute_metric)



from collections import OrderedDict

from transformers.modeling_roberta import RobertaConfig, RobertaModel, RobertaForMaskedLM, RobertaForMultipleChoice, \
    RobertaForQuestionAnswering, RobertaForSequenceClassification, RobertaForTokenClassification
from transformers.tokenization_roberta import RobertaTokenizer, RobertaTokenizerFast

from ..model_base import TaskModels, ModelType, ModelTaskType


class Roberta_Task_Models(TaskModels):
    MODEL_TYPE = ModelType.roberta
    CONFIG = RobertaConfig
    MODEL_PATHS = {
        "cn": ["clue/roberta_chinese_clue_tiny",
               "clue/roberta_chinese_3L312_clue_tiny",
               "roberta_chinese_3L768_clue_tiny",
               "clue/roberta_chinese_pair_tiny",
               "lonePatient/roberta_chinese_clue_tiny",
               "clue/roberta_chinese_clue_base",
               "hfl/chinese-roberta-wwm-ext",
               "clue/roberta_chinese_large",
               "chinese-roberta-wwm-ext-large",
               "clue/roberta_chinese_clue_large",
               "clue/roberta_chinese_pair_large"
               ]}
    MODEL_CLASSES = OrderedDict([
        (ModelTaskType.base, RobertaModel),
        (ModelTaskType.lm_head, RobertaForMaskedLM),
        (ModelTaskType.seq_cls, RobertaForSequenceClassification),
        (ModelTaskType.token_cls, RobertaForTokenClassification),
        (ModelTaskType.qa, RobertaForQuestionAnswering),
        (ModelTaskType.multi_choice, RobertaForMultipleChoice)
    ])
    TOKENIZERS = OrderedDict([
        ('default', RobertaTokenizer),
        ('fast', RobertaTokenizerFast)
    ])



import dataclasses
import json
import logging
from dataclasses import dataclass, field
from typing import Any, Dict, Optional, Tuple

from transformers import is_torch_available
from transformers.file_utils import torch_required, cached_property

if is_torch_available():
    import torch

try:
    import torch_xla.core.xla_model as xm

    _has_tpu = True
except ImportError:
    _has_tpu = False


@torch_required
def is_tpu_available():
    return _has_tpu


logger = logging.getLogger(__name__)


@dataclass
class TrainingArguments:
    """
    TrainingArguments is the subset of the arguments we use in our example scripts
    **which relate to the training loop itself**.

    Using `HfArgumentParser` we can turn this class
    into argparse arguments to be able to specify them on
    the command line.
    """

    output_dir: str = field(default='models/finetuning',
                            metadata={
                                "help": "The output directory where the model predictions and checkpoints will be written."}
                            )
    overwrite_output_dir: bool = field(
        default=False,
        metadata={
            "help": (
                "Overwrite the content of the output directory."
                "Use this to continue training if output_dir points to a checkpoint directory."
            )
        },
    )

    do_train: bool = field(default=False, metadata={"help": "Whether to run training."})
    do_eval: bool = field(default=False, metadata={"help": "Whether to run eval on the dev set."})
    do_predict: bool = field(default=False, metadata={"help": "Whether to run predictions on the test set."})
    evaluate_during_training: bool = field(
        default=True, metadata={"help": "Run evaluation during training at each logging step."},
    )

    per_gpu_train_batch_size: int = field(default=16, metadata={"help": "Batch size per GPU/CPU for training."})
    per_gpu_eval_batch_size: int = field(default=16, metadata={"help": "Batch size per GPU/CPU for evaluation."})
    gradient_accumulation_steps: int = field(
        default=1,
        metadata={"help": "Number of updates steps to accumulate before performing a backward/update pass."},
    )

    learning_rate: float = field(default=5e-5, metadata={"help": "The initial learning rate for Adam."})
    weight_decay: float = field(default=0.0, metadata={"help": "Weight decay if we apply some."})
    adam_epsilon: float = field(default=1e-8, metadata={"help": "Epsilon for Adam optimizer."})
    max_grad_norm: float = field(default=1.0, metadata={"help": "Max gradient norm."})

    num_train_epochs: float = field(default=3.0, metadata={"help": "Total number of training epochs to perform."})

    max_steps: int = field(
        default=-1,
        metadata={"help": "If > 0: set total number of training steps to perform. Override num_train_epochs."},
    )

    eval_max_steps: int = field(
        default=-1,
        metadata={"help": "If > 0: set total number of training steps to perform."},
    )
    warmup_steps: int = field(default=0, metadata={"help": "Linear warmup over warmup_steps."})

    logging_dir: Optional[str] = field(default=None, metadata={"help": "Tensorboard log dir."})
    logging_first_step: bool = field(default=False, metadata={"help": "Log and eval the first global_step"})
    logging_steps: int = field(default=500, metadata={"help": "Log every X updates steps."})
    save_steps: int = field(default=500, metadata={"help": "Save checkpoint every X updates steps."})
    save_total_limit: Optional[int] = field(
        default=None,
        metadata={
            "help": (
                "Limit the total amount of checkpoints."
                "Deletes the older checkpoints in the output_dir. Default is unlimited checkpoints"
            )
        },
    )
    no_cuda: bool = field(default=False, metadata={"help": "Do not use CUDA even when it is available"})
    seed: int = field(default=42, metadata={"help": "random seed for initialization"})

    fp16: bool = field(
        default=False,
        metadata={"help": "Whether to use 16-bit (mixed) precision (through NVIDIA apex) instead of 32-bit"},
    )
    fp16_opt_level: str = field(
        default="O1",
        metadata={
            "help": (
                "For fp16: Apex AMP optimization level selected in ['O0', 'O1', 'O2', and 'O3']."
                "See details at https://nvidia.github.io/apex/amp.html"
            )
        },
    )
    local_rank: int = field(default=-1, metadata={"help": "For distributed training: local_rank"})

    tpu_num_cores: Optional[int] = field(
        default=None, metadata={"help": "TPU: Number of TPU cores (automatically passed by launcher script)"}
    )
    tpu_metrics_debug: bool = field(default=False, metadata={"help": "TPU: Whether to print debug metrics"})

    @property
    def train_batch_size(self) -> int:
        return self.per_gpu_train_batch_size * max(1, self.n_gpu)

    @property
    def eval_batch_size(self) -> int:
        return self.per_gpu_eval_batch_size * max(1, self.n_gpu)

    @cached_property
    @torch_required
    def _setup_devices(self) -> Tuple["torch.device", int]:
        logger.info("PyTorch: setting up devices")
        if self.no_cuda:
            device = torch.device("cpu")
            n_gpu = 0
        elif is_tpu_available():
            device = xm.xla_device()
            n_gpu = 0
        elif self.local_rank == -1:
            # if n_gpu is > 1 we'll use nn.DataParallel.
            # If you only want to use a specific subset of GPUs use `CUDA_VISIBLE_DEVICES=0`
            device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
            n_gpu = torch.cuda.device_count()
        else:
            # Here, we'll use torch.distributed.
            # Initializes the distributed backend which will take care of sychronizing nodes/GPUs
            torch.distributed.init_process_group(backend="nccl")
            device = torch.device("cuda", self.local_rank)
            n_gpu = 1
        return device, n_gpu

    @property
    @torch_required
    def device(self) -> "torch.device":
        return self._setup_devices[0]

    @property
    @torch_required
    def n_gpu(self):
        return self._setup_devices[1]

    def to_json_string(self):
        """
        Serializes this instance to a JSON string.
        """
        return json.dumps(dataclasses.asdict(self), indent=2)

    def to_sanitized_dict(self) -> Dict[str, Any]:
        """
        Sanitized serialization to use with TensorBoard’s hparams
        """
        d = dataclasses.asdict(self)
        valid_types = [bool, int, float, str]
        if is_torch_available():
            valid_types.append(torch.Tensor)
        return {k: v if type(v) in valid_types else str(v) for k, v in d.items()}

    def validate(self):
        if self.evaluate_during_training:
            self.do_eval = True

    def train(self):
        self.do_train = True
        self.do_eval = True
        self.evaluate_during_training = True



from transformers import BertConfig, BertTokenizer

from ai_transformersx import *
from ai_transformersx.examples import ExampleManagement, ExampleTaskBase
from ai_transformersx.examples.main import manager


class TestTransformersxTask:
    task_args: TaskArguments = parse_tasks_args()

    def test_parse_arg(self):
        assert task_args.model_args.unit_test == False

    def test_task_model(self):
        self.task_args.model_args.unit_test = True
        taskModel: TaskModel = TaskModel(self.task_args.model_args)
        assert isinstance(taskModel.config, (BertConfig))
        assert isinstance(taskModel.tokenizer, (BertTokenizer))

    def test_parse_task_arguments(self):
        manager = ExampleManagement()
        argument_objs = manager._build_arguments()
        args = ["news", "--action=train", "--processor-args.data-dir=/test/test"]
        task_name, arguments = ComplexArguments(argument_objs).parse(args)
        assert task_name == "news"
        assert arguments.processor_args.data_dir == "/test/test"

    def test_start_example_task(self):
        manager.start_example_task(["news", "--action=train", "--processor-args.data-dir=/test/test"], True)



from collections import OrderedDict

from transformers.modeling_xlnet import XLNetConfig, XLNetModel, XLNetForMultipleChoice, XLNetForQuestionAnswering, \
    XLNetForQuestionAnsweringSimple, XLNetForSequenceClassification, XLNetForTokenClassification, XLNetLMHeadModel
from transformers.tokenization_xlnet import XLNetTokenizer

from ..model_base import TaskModels, ModelType, ModelTaskType


class XLNet_Task_Models(TaskModels):
    MODEL_TYPE = ModelType.xlnet
    CONFIG = XLNetConfig
    MODEL_PATHS = {
        "cn": ["clue/roberta_chinese_clue_tiny",
               "clue/roberta_chinese_3L312_clue_tiny",
               "roberta_chinese_3L768_clue_tiny",
               "clue/roberta_chinese_pair_tiny",
               "lonePatient/roberta_chinese_clue_tiny",
               "clue/roberta_chinese_clue_base",
               "hfl/chinese-roberta-wwm-ext",
               "clue/roberta_chinese_large",
               "chinese-roberta-wwm-ext-large",
               "clue/roberta_chinese_clue_large",
               "clue/roberta_chinese_pair_large"
               ]}
    MODEL_CLASSES = OrderedDict([
        (ModelTaskType.base, XLNetModel),
        (ModelTaskType.lm_head, XLNetLMHeadModel),
        (ModelTaskType.seq_cls, XLNetForSequenceClassification),
        (ModelTaskType.token_cls, XLNetForTokenClassification),
        (ModelTaskType.qa, XLNetForQuestionAnswering),
        (ModelTaskType.qa_s, XLNetForQuestionAnsweringSimple),
        (ModelTaskType.multi_choice, XLNetForMultipleChoice)
    ])
    TOKENIZERS = OrderedDict([
        ('default', XLNetTokenizer)
    ])



from dataclasses import dataclass, fields
from ai_harness import harnessutils as aiutils

log = aiutils.getLogger('task')


@dataclass(frozen=True)
class ModelType:
    bert: str = "bert"
    albert: str = "albert"
    electra: str = "electra"
    roberta: str = "robrerta"
    xlnet: str = "xlnet"

    @staticmethod
    def names():
        return [f.name for f in fields(ModelType)]


@dataclass
class ModelMode:
    classification: str = 'classification'
    regression: str = 'regression'

    @staticmethod
    def names():
        return [f.name for f in fields(ModelMode)]


@dataclass
class ModelTaskType:
    base: str = 'base'
    pretrain: str = 'pretrain'
    lm_head: str = 'lm_head'
    qa: str = 'qa'
    qa_s: str = 'qa_s'
    seq_cls: str = 'seq_cls'
    token_cls: str = 'token_cls'
    multi_choice: str = 'multi_choice'
    next_seq: str = 'next_seq'

    @staticmethod
    def names():
        return [f.name for f in fields(ModelTaskType)]


class TaskModel:
    def __init__(self, model_type, config, model_path, model_class, tokenizer):
        self.model_type = model_type
        self.config = config
        self.model_path = model_path
        self.model_class = model_class
        self.tokenizer = tokenizer

    def load(self, **kwargs):
        config = self.config.from_pretrained(
            self.model_path,
            num_labels=kwargs['num_labels']
        )
        tokenizer = self.tokenizer.from_pretrained(self.model_path)
        unit_test = kwargs['unit_test']
        if not unit_test:
            model = self.model_class.from_pretrained(
                self.model_path,
                config=config
            )
        else:
            model = None
        return config, tokenizer, model


class TaskModels:
    MODEL_TYPE = None
    MODEL_PATHS = {}
    MODEL_CLASSES = {}
    TOKENIZERS = {}
    CONFIG = None

    def _model_class(self, model_path, model_task_type, tokenizer_name='default', language='cn',
                     ignore_not_exists=False):
        if not model_path in self.MODEL_PATHS[language]:
            if not ignore_not_exists:
                raise ValueError(
                    "Cannot find the model path {} for language {},model_type={}".format(model_path, language,
                                                                                         self.MODEL_TYPE))
            return None

        tokenizer = self.TOKENIZERS[tokenizer_name]
        if not tokenizer:
            if not ignore_not_exists:
                raise ValueError(
                    "Cannot find the tokenizer with name {},model_type={}.".format(tokenizer_name, self.MODEL_TYPE))
            return None

        model_class = self.MODEL_CLASSES[model_task_type]
        if not model_class:
            log.warn(
                "Cannot find the model class of task type {},model_type={}.".format(model_task_type, self.MODEL_TYPE))

        return model_class, tokenizer

    def all_base_models(self, language=None):
        pass

    def task_model(self, model_path, model_task_type, tokenizer_name='default', language='cn',
                   ignore_not_exists=False) -> TaskModel:
        model_cls = self._model_class(model_path, model_task_type, tokenizer_name, language, ignore_not_exists)
        if not model_cls and ignore_not_exists:
            return None
        return TaskModel(self.MODEL_TYPE, self.CONFIG, model_path,
                         *model_cls)



from collections import OrderedDict

from .electra.modelingx_electra import ElectraForSequenceClassificationX
from .bert.models import Bert_Task_Models
from .albert.models import Albert_Task_Models
from .electra.models import Electra_Task_Models
from .roberta.models import Roberta_Task_Models
from .xlnet.models import XLNet_Task_Models
from .model_base import ModelType, ModelTaskType, ModelMode

ALL_TASK_MODELS = OrderedDict([
    (ModelType.albert, Albert_Task_Models()),
    (ModelType.bert, Bert_Task_Models()),
    (ModelType.roberta, Roberta_Task_Models()),
    (ModelType.electra, Electra_Task_Models()),
    (ModelType.xlnet, XLNet_Task_Models())
])

ALL_TASK_MODEL_PATHS = [task_model.MODEL_PATHS for task_model in ALL_TASK_MODELS.values()]


def task_model(model_path, model_task_type, model_type=None, tokenizer_name='default', language='cn'):
    models = ALL_TASK_MODELS.get(model_type)
    if models:
        return models.task_model(model_path, model_task_type, tokenizer_name, language)
    return _task_model_no_type(model_path, model_task_type, tokenizer_name, language)


def _task_model_no_type(model_path, model_task_type, tokenizer_name='default', language='cn'):
    for models in ALL_TASK_MODELS.values():
        model = models.task_model(model_path, model_task_type, tokenizer_name, language, ignore_not_exists=True)
        if model:
            return model
    raise ValueError("Cannot find models with model path {}".format(model_path))
